#include <proto.h>

#include "gtradiogroup.h"
#include "gttoggle.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtRadioGroup class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init radiogroup.
*
\************************************************************************/
GtRadioGroup *GtMkRadioGroup(char *n,GtManager *p,GtTag *tgs)
{
	GtRadioGroup *obj;
	
	if(!(obj=new GtRadioGroup()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtRadioGroup object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init radiogroup.
*
\************************************************************************/
GtRadioGroup *GtMkRadioGroup(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtRadioGroup *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkRadioGroup(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty radiogroup.
*
\************************************************************************/
GtRadioGroup::GtRadioGroup(void)
{
	active=NULL;
	rtype=GTRT_ZERO;
}

/************************************************************************\
*
* Setup radiogroup.
*
\************************************************************************/
sbool GtRadioGroup::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtGroup::Init(n,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_RadioType:
				rtype=t->d.val;
				if(rtype>GTRT_LAST_NR)
					rtype=GTRT_ZERO;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Set radiogroup attributes.
*
\************************************************************************/
sbool GtRadioGroup::Set(GtTag *tgs)
{
	GtTag *t;
	
	if(!GtGroup::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_RadioType:
				rtype=t->d.val;
				if(rtype>GTRT_LAST_NR)
					rtype=GTRT_ZERO;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Get radiogroup attributes.
*
\************************************************************************/
void GtRadioGroup::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtGroup::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_RadioType:
				*ptr=rtype;
				break;
			case GT_CurrentOn:
				*ptr=(long)active;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtRadioGroup::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtRadioGroup")==0)
		return(TRUE);
	else
		return(GtGroup::IsSubClassOf(cn));
}

/************************************************************************\
*
* Request from child to toggle on/off.
*
\************************************************************************/
sbool GtRadioGroup::ChangeReq(GtToggle *t,sbool on)
{
	if((t==active) && on && (rtype==GTRT_ONE))
		return(TRUE);
	if((t==active) && on)
	{
		active=NULL;
		return(FALSE);
	}
	if(!on)
	{
		if(active)
			active->SetV(GT_RadioOnOff,FALSE,GT_DONE);
		active=t;
	}
	return(!on);
}

/************************************************************************\
*
* Request from child to set on/off.
*
\************************************************************************/
sbool GtRadioGroup::SetReq(GtToggle *t,sbool on)
{
	if((t==active) && !on && (rtype==GTRT_ONE))
		return(TRUE);
	if((t==active) && !on)
	{
		active=NULL;
		return(FALSE);
	}
	if(on)
	{
		if(active)
			active->SetV(GT_RadioOnOff,FALSE,GT_DONE);
		active=t;
	}
	return(on);
}
